﻿using Kivii.Controls;
using Kivii.Zip.Zlib;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Kivii.Sample.ImageUploader
{
    public partial class FrmImageEditor : Form
    {
        private float _zoom = 1f;
        public FrmImageEditor()
        {
            InitializeComponent();
        }

        public Bitmap backgroundImage;

        public Bitmap ImgEdited;

        //图像裁剪
        public Bitmap GetPartOfImage(Bitmap sourceImg, int width, int height, int offsetX, int offsetY)
        {
            Bitmap sourceBitmap = sourceImg;

            Bitmap rtns = new Bitmap(width, height);
            using (Graphics g = Graphics.FromImage(rtns))
            {
                Rectangle resultRectangle = new Rectangle(0, 0, width, height);
                Rectangle sourceRectangle = new Rectangle(0 + offsetX, 0 + offsetY, width, height);

                g.DrawImage(sourceBitmap, resultRectangle, sourceRectangle, GraphicsUnit.Pixel);
            }
            return rtns;
        }

        private void timeImageEditor_Tick(object sender, EventArgs e)
        {
            FrmMain main = (FrmMain)this.Owner;
            main.setBackgroundImage(ImgEdited);
            this.Close();
        }

        private void FrmImageEditor_Load(object sender, EventArgs e)
        {
            this.plImage.BackgroundImage = backgroundImage;
            this.plImage.BackgroundImageLayout = ImageLayout.Zoom;
            ImgEdited = backgroundImage;
            resizePictureBox();
        }
        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        private void btnSave_Click(object sender, EventArgs e)
        {
            System.Drawing.Point p = new System.Drawing.Point(imgResizer.Location.X + imgResizer.blank, imgResizer.Location.Y + imgResizer.blank);
            var imgArea = imgResizer.area;
            Rectangle area = new Rectangle(p, imgArea.Size);
            ImgEdited = GetPartOfImage(backgroundImage, area.Width, area.Height, p.X, p.Y);
            
            Bitmap bmpFluffy = ImgEdited;
            //new Bitmap(after);

            Rectangle r = new Rectangle(System.Drawing.Point.Empty, bmpFluffy.Size);

            using (Bitmap bmpMask = new Bitmap(r.Width, r.Height))

            using (Graphics g = Graphics.FromImage(bmpMask))

            using (GraphicsPath path = createRoundRect(r.X, r.Y, r.Width, r.Height,0))

            using (Brush brush = createFluffyBrush(path,new float[] { 0.0f, 0.1f, 1.0f },new float[] { 0.0f, 0.95f, 1.0f }))
            {
                g.FillRectangle(Brushes.Black, r);
                g.SmoothingMode = SmoothingMode.HighQuality;
                g.FillPath(brush, path);
                transferOneARGBChannelFromOneBitmapToAnother(bmpMask,bmpFluffy,ChannelARGB.Blue,ChannelARGB.Alpha);
            }

            Graphics drawBk = Graphics.FromImage(backgroundImage);
            drawBk.Clear(Color.FromKnownColor(KnownColor.Control));
            drawBk.DrawImage(ImgEdited, area);

            Graphics winGph = Graphics.FromHwnd(this.Handle);
            winGph.DrawImage(backgroundImage, new System.Drawing.Point(0, 0));

            this.plImage.BackgroundImage = backgroundImage;

            imgResizer.Visible = false;

            this.plImage.Refresh();
            timeImageEditor.Enabled = true;
        }

        #region 窗口大小发生变化
        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            resizePictureBox();
        }
        private void resizePictureBox()
        {
            if (plImage.BackgroundImage == null) return;
            int width, height;
            if (_zoom == 0f)
            {
                float wzoom = (float)plImage.Width / (float)plImage.BackgroundImage.Width;
                float hzoom = (float)plImage.Height / (float)plImage.BackgroundImage.Height;
                _zoom = wzoom < hzoom ? wzoom : hzoom;
            }
            else if (_zoom > 5f) _zoom = 5f;
            else if (_zoom < 0.1f) _zoom = 0.1f;
            width = (int)((float)plImage.BackgroundImage.Width * _zoom);
            height = (int)((float)plImage.BackgroundImage.Height * _zoom);
            plImage.Width = width;
            plImage.Height = height;
            int x, y;
            if (plImage.Width <= width) x = 0;
            else x = (plImage.Width - width) / 2;
            if (plImage.Height <= height) y = 0;
            else y = (plImage.Height - height) / 2;
            plImage.Location = new System.Drawing.Point(x, y);

            // 计算控件在父容器中居中的位置
            int xi = (imgResizer.Parent.Width - imgResizer.Width) / 2;
            int yi = (imgResizer.Parent.Height - imgResizer.Height) / 2;
            imgResizer.Location= new System.Drawing.Point(xi, yi);

        }
        #endregion

        public static GraphicsPath createRoundRect(int x, int y, int width, int height, int radius)
        {

            GraphicsPath gp = new GraphicsPath();

            if (radius == 0)

                gp.AddRectangle(new Rectangle(x, y, width, height));

            else
            {

                gp.AddLine(x + radius, y, x + width - radius, y);

                gp.AddArc(x + width - radius, y, radius, radius, 270, 90);

                gp.AddLine(x + width, y + radius, x + width, y + height - radius);

                gp.AddArc(x + width - radius, y + height - radius, radius, radius, 0, 90);

                gp.AddLine(x + width - radius, y + height, x + radius, y + height);

                gp.AddArc(x, y + height - radius, radius, radius, 90, 90);

                gp.AddLine(x, y + height - radius, x, y + radius);

                gp.AddArc(x, y, radius, radius, 180, 90);

                gp.CloseFigure();

            }

            return gp;

        }
        public static Brush createFluffyBrush(GraphicsPath gp, float[] blendPositions, float[] blendFactors)
        {

            PathGradientBrush pgb = new PathGradientBrush(gp);

            //Blend blend = new Blend();

            //blend.Positions = blendPositions;

            //blend.Factors = blendFactors;

            //pgb.Blend = blend;

            //pgb.CenterColor = Color.White;

            //pgb.SurroundColors = new Color[] { Color.Black };

            return pgb;

        }
        public enum ChannelARGB
        {

            Blue = 0,

            Green = 1,

            Red = 2,

            Alpha = 3

        }

        public static void transferOneARGBChannelFromOneBitmapToAnother(Bitmap source, Bitmap dest, ChannelARGB sourceChannel, ChannelARGB destChannel)
        {
            if (source.Size != dest.Size)

                throw new ArgumentException();

            Rectangle r = new Rectangle(System.Drawing.Point.Empty, source.Size);

            BitmapData bdSrc = source.LockBits(r, ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);

            BitmapData bdDst = dest.LockBits(r, ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);

            unsafe
            {

                byte* bpSrc = (byte*)bdSrc.Scan0.ToPointer();

                byte* bpDst = (byte*)bdDst.Scan0.ToPointer();

                bpSrc += (int)sourceChannel;

                bpDst += (int)destChannel;

                for (int i = r.Height * r.Width; i > 0; i--)
                {

                    *bpDst = *bpSrc;

                    bpSrc += 4;

                    bpDst += 4;

                }

            }

            source.UnlockBits(bdSrc);

            dest.UnlockBits(bdDst);

        }

        private void FrmImageEditor_Resize(object sender, EventArgs e)
        {
            resizePictureBox();
        }
    }
}
